package Fairness.Nodes.NodeBehaviors;

import Fairness.Nodes.Node;
import Fairness.Nodes.NodePacket;

import java.util.HashMap;
import java.util.Map;

/**
 * DebtMaintenanceBehavior applies debts to children who receives a packet and applies credits 
 * to its immediate parent
 *
 * @author Jason Winnebeck
 */
public class DebtMaintenanceBehavior implements NodeBehavior {
    private final Map<Node, Debt> debts = new HashMap<Node, Debt>();
    private double threshold = Double.MAX_VALUE;

    /**
     * Allows setting of a single behavior-specific parameter.
     *
     * @param param non-null String
     */
    public void setParameter( String param ) {
        threshold = Double.parseDouble( param );
    }

    /**
     * Asks the node behavior if the given Node is allowed to be a child of the Node this behavior
     * is for.
     *
     * @param myNode         non-null Node to which this behavior belongs.
     * @param candidateChild non-null Node that is not equal to myNode.
     *
     * @return true if the child can be added, or false otherwise.
     */
    public boolean allowChild( Node myNode, Node candidateChild ) {
        return true;
    }

    /**
     * Returns true if this node behavior is currently allowing the Node to send to the given
     * target.
     *
     * @param myNode non-null Node to which this behavior belongs.
     * @param target non-null Node that is not equal to myNode.
     */
    public boolean allowForward( Node myNode, Node target ) {
        Debt d = debts.get( target );
        return ( d == null ) || d.debt < threshold;
    }

    /**
     * Event to the node behavior for when a packet is received by the Node.
     *
     * @param myNode non-null Node to which this behavior belongs.
     * @param packet non-null NodePacket destined for this behavior's Node.
     */
    public void receivedPacket( Node myNode, NodePacket packet ) {
        if ( packet.getSource() != null && packet.isLive() ) {
            changeDebt( packet.getSource(), -1 );
        }
    }

    /**
     * Event called when a Node sends a packet to a child.
     *
     * @param myNode      non-null Node to which this behavior belongs.
     * @param destination non-null Node where the packet is going.
     * @param packet      non-null NodePacket destined for this behavior's Node.
     */
    public void sentPacket( Node myNode, Node destination, NodePacket packet ) {
        changeDebt( destination, 1 );
    }

    /**
     * Returns true if this behavior has confidence information on any Node
     */
    public boolean hasConfidenceInformation() {
        return false;
    }

    /**
     * Returns true if this behavior has debt levels for any Node
     */
    public boolean hasDebtLevelInformation() {
        return true;
    }

    /**
     * Returns this behavior's confidence of the given Node, or null if there is no opinion.
     *
     * @param n non-null Node to get information about
     *
     * @return nullable Double
     */
    public Double getConfidence( Node n ) {
        return null;
    }

    /**
     * Returns this behavior's debt level measurement of the given Node, or null if there is no
     * opinion.
     *
     * @param n non-null Node to get information about
     *
     * @return nullable Double
     */
    public Double getDebtLevel( Node n ) {
        final Debt debt = debts.get( n );
        return ( debt != null ) ? (double) debt.debt : null;
    }
    /**
     * Changes the value of debts for a given node
     * opinion.
     *
     * @param node non-null Node to get its debt information 
     * @param change   value to be changed
     */
    private void changeDebt( Node node, int change ) {
        Debt d = debts.get( node );
        if ( d == null ) {
            d = new Debt();
            debts.put( node, d );
        }
        d.debt += change;
    }
    /**
     *Set the initial value of Debt to zero
     */
    private static class Debt {
        private int debt = 0;
    }
}
