package Fairness.Nodes.NodeBehaviors;

import Fairness.Nodes.Node;
import Fairness.Nodes.NodePacket;

/**
 * NodeBehavior class provides an interface for the different node behaviors such as allowing to
 * forward and receive packets and also contains information about the debts and confidence of nodes.
 * It allows implementation of a freeloader or a freeloader behavior for different behavior types such
 * AncestorRatingBehavior and DebtMaintenanceBehavior
 *
 * @author Jason Winnebeck
 */
public interface NodeBehavior {
    /**
     * Allows setting of a single behavior-specific parameter.
     *
     * @param param non-null String
     */
    public void setParameter( String param );

    /**
     * Asks the node behavior if the given Node is allowed to be a child of the Node this behavior
     * is for.
     *
     * @param myNode         non-null Node to which this behavior belongs.
     * @param candidateChild non-null Node that is not equal to myNode.
     *
     * @return true if the child can be added, or false otherwise.
     */
    public boolean allowChild( Node myNode, Node candidateChild );

    /**
     * Returns true if this node behavior is currently allowing the Node to send to the given
     * target.
     *
     * @param myNode non-null Node to which this behavior belongs.
     * @param target non-null Node that is not equal to myNode.
     */
    public boolean allowForward( Node myNode, Node target );

    /**
     * Event to the node behavior for when a packet is received by the Node.
     *
     * @param myNode non-null Node to which this behavior belongs.
     * @param packet non-null NodePacket destined for this behavior's Node.
     */
    public void receivedPacket( Node myNode, NodePacket packet );

    /**
     * Event called when a Node sends a packet to a child.
     *
     * @param myNode      non-null Node to which this behavior belongs.
     * @param destination non-null Node where the packet is going.
     * @param packet      non-null NodePacket destined for this behavior's Node.
     */
    public void sentPacket( Node myNode, Node destination, NodePacket packet );

    /**
     * Returns true if this behavior has confidence information on any Node
     */
    public boolean hasConfidenceInformation();

    /**
     * Returns true if this behavior has debt levels for any Node
     */
    public boolean hasDebtLevelInformation();

    /**
     * Returns this behavior's confidence of the given Node, or null if there is no opinion.
     *
     * @param n non-null Node to get information about
     * @return nullable Double
     */
    public Double getConfidence( Node n );

    /**
     * Returns this behavior's debt level measurement of the given Node, or null if there is no
     * opinion.
     *
     * @param n non-null Node to get information about
     * @return nullable Double
     */
    public Double getDebtLevel( Node n );
}
