package Fairness;

import java.util.Random;

/**
 * NumberUtil provides some numeric utilities. NumberUtil cannot be instantiated.
 *
 * @author Jason Winnebeck
 */
public class NumberUtil {
    private NumberUtil() {
    }

    /**
     * Returns a single hex digit from an integer. Each digit, therefore, is 4 bits wide
     *
     * @param num   integer
     * @param digit digit (0 through 7) to return; 0 is the most significant
     *
     * @return digit (0 through 15)
     */
    public static int getDigit( int num, int digit ) {
        if ( digit < 0 || digit > 7 )
          throw new IllegalArgumentException( "Parameter digit to NumberUtil.getDigit must be within the range 0-7, inclusive; its value is " + digit + '.' );

        int shift = 32 - (digit + 1) * 4;
        return ( num >>> shift ) & 0xF;
    }

    /**
     * Returns the size of the common hex digit prefix of two integers. For example 0xAA012345 and
     * 0xAA543210 share AA as a prefix, so the return value would be 2.
     *
     * @return size of shared prefix, 0 through 8.
     */
    public static int numMatchingPrefix( int a, int b ) {
        int currDigit = 0;
        while ( currDigit < 8 && getDigit( a, currDigit ) == getDigit( b, currDigit ) )
            currDigit++;
        return currDigit;
    }

    /**
     * Returns a number that has a fixed prefix but all digits after the prefix are chosen at
     * random.
     *
     * @param prefix       prefix to have (such as 0xA)
     * @param prefixDigits number of digits in prefix (such as 1)
     * @param r            non-null source of randomness
     *
     * @return             random number that starts with the prefix in highest-order digits.
     */
    public static int getRandomWithPrefix( int prefix, int prefixDigits, Random r ) {
        if ( prefixDigits < 0 || prefixDigits > 8 )
          throw new IllegalArgumentException( "Parameter prefixDigits to NumberUtil.getRandomWithPrefix must be within the range 0-8, inclusive; its value is " + prefixDigits + '.' );

        int numBits = 32 - prefixDigits * 4;
        int ret = r.nextInt( 1 << numBits );
        return ret | ( prefix << numBits );
    }
}
