package Fairness;

import Fairness.Trees.TreeManager;
import org.gillius.Util.CheckUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Collection;

/**
 * TreeReconstructionEvent invokes the {@link TreeManager#rebuildTree()} method at a given rate.
 * This event is "perpetual" in that it will always reschedule itself when it fires. When the tree
 * is rebuilt, all NodeTransferEvent events are cancelled to drop all packets in the network.
 * <p/>
 * Also, a collection of NodeMonitor objects are held in this event, and when the event fires, all
 * of the monitors get marked.
 *
 * @author Jason Winnebeck
 */
public class TreeReconstructionEvent implements SimulationEvent {
    private final TreeManager manager;
    private final double rebuildRate;

    private final List<NodeMonitor> monitors;

    /**
     * Constructs a new TreeReconstructionEvent to call rebuildTree on the given manager every
     * 'rebuildRate' seconds.
     */
    public TreeReconstructionEvent( TreeManager manager, Collection<NodeMonitor> monitors,
                                    double rebuildRate ) {
        CheckUtils.checkNull( monitors, "monitors" );

        this.manager = manager;
        this.monitors = new ArrayList<NodeMonitor>( monitors );
        this.rebuildRate = rebuildRate;
    }

    /**
     * Performs the event that is occuring within a {@link Simulation}. If an event wishes
     * to re-execute later, it may schedule itself again using the provided Simulation reference.
     *
     * @param sim  non-null Simulation
     * @param time simulation time, exactly equal to the result of {@link Simulation#getTime()}
     */
    public void perform( Simulation sim, double time ) {
        manager.rebuildTree();
        sim.cancelEvents( NodeTransferEvent.class );
        for ( NodeMonitor monitor : monitors )
            monitor.mark();

        sim.scheduleEvent( time + rebuildRate, this );
    }
}
